import { PlatformAccessory } from 'homebridge';
import { EufySecurityPlatform } from '../platform';
import { DeviceAccessory } from './Device';
import { Lock } from 'eufy-security-client';
/**
 * LockAccessory Class
 *
 * This class represents a lock accessory within a home automation system. It is designed to
 * integrate smart locks into the system, register appropriate HomeKit characteristics, and provide
 * functionality for controlling and monitoring the lock's status.
 *
 * @class LockAccessory
 * @extends DeviceAccessory
 */
export declare class LockAccessory extends DeviceAccessory {
    /**
     * Constructor for LockAccessory.
     *
     * @param {EufySecurityPlatform} platform - The platform instance managing accessories.
     * @param {PlatformAccessory} accessory - The platform-specific accessory.
     * @param {Lock} device - The lock device being represented.
     */
    constructor(platform: EufySecurityPlatform, accessory: PlatformAccessory, device: Lock);
    /**
     * Initializes characteristics for the Lock Management Service.
     */
    private initLockManagementService;
    /**
     * Initializes characteristics for the Lock Mechanism Service.
     */
    private initLockMechanismService;
    /**
     * Gets the lock status and maps it to HomeKit lock states.
     */
    private getLockStatus;
    /**
     * Sets the lock target state asynchronously.
     */
    private setLockTargetState;
    /**
     * Converts lock status codes to corresponding HomeKit lock states.
     */
    private convertLockStatusCode;
}
//# sourceMappingURL=LockAccessory.d.ts.map